@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -97,14 +97,21 @@ template<UnsignedInt dimensions, class T> class Range {
         }
 
         /**
-         * @brief Construct zero range
+         * @brief Default constructor
          *
-         * Construct zero-size range positioned at origin.
+         * Equivalent to @ref Range(ZeroInitT).
          */
-        constexpr /*implicit*/ Range(ZeroInitT = ZeroInit) noexcept: Range<dimensions, T>{ZeroInit, typename std::conditional<dimensions == 1, void*, ZeroInitT*>::type{}} {}
+        constexpr /*implicit*/ Range() noexcept: Range<dimensions, T>{ZeroInit, typename std::conditional<dimensions == 1, void*, ZeroInitT*>::type{}} {}
+
+        /**
+         * @brief Construct a zero range
+         *
+         * Construct a zero-size range positioned at origin.
+         */
+        constexpr explicit Range(ZeroInitT) noexcept: Range<dimensions, T>{ZeroInit, typename std::conditional<dimensions == 1, void*, ZeroInitT*>::type{}} {}
 
         /** @brief Construct without initializing the contents */
-        explicit Range(NoInitT) noexcept: Range<dimensions, T>{NoInit, typename std::conditional<dimensions == 1, void*, NoInitT*>::type{}} {}
+        explicit Range(Magnum::NoInitT) noexcept: Range<dimensions, T>{Magnum::NoInit, typename std::conditional<dimensions == 1, void*, Magnum::NoInitT*>::type{}} {}
 
         /** @brief Construct a range from minimal and maximal coordinates */
         constexpr /*implicit*/ Range(const VectorType& min, const VectorType& max) noexcept: _min{min}, _max{max} {}
@@ -115,9 +122,10 @@ template<UnsignedInt dimensions, class T> class Range {
          * Useful in combination with e.g. @ref minmax(), here for example to
          * calculate bounds of a triangle:
          *
-         * @snippet MagnumMath.cpp Range-construct-minmax
+         * @snippet MagnumMath.cpp Range-construct-minmax3D
          *
-         * @todo std::pair constructors are not constexpr in C++11, make it so in C++14 */
+         * @todo std::pair constructors are not constexpr in C++11, make it so in C++14
+         */
         /*implicit*/ Range(const std::pair<VectorType, VectorType>& minmax) noexcept:
             _min{minmax.first}, _max{minmax.second} {}
 
@@ -218,7 +226,7 @@ template<UnsignedInt dimensions, class T> class Range {
         }
 
         /**
-         * @brief Padded rage
+         * @brief Padded range
          *
          * Translates the minimal and maximal coordinates by given amount.
          * Center remains the same.
@@ -298,8 +306,8 @@ template<UnsignedInt dimensions, class T> class Range {
 
         /* Called from Range(NoInit), either using the NoInit constructor (if
            available) or not doing anything */
-        explicit Range(NoInitT, NoInitT*) noexcept: _min{NoInit}, _max{NoInit} {}
-        explicit Range(NoInitT, void*) noexcept {}
+        explicit Range(Magnum::NoInitT, Magnum::NoInitT*) noexcept: _min{Magnum::NoInit}, _max{Magnum::NoInit} {}
+        explicit Range(Magnum::NoInitT, void*) noexcept {}
 
         /* Called from data(), always returning a T* */
         constexpr const VectorType* dataInternal(void*) const { return &_min; }
@@ -327,6 +335,9 @@ template<UnsignedInt dimensions, class T> class Range {
     }                                                                       \
     Type<T> scaled(const VectorType<T>& scaling) const {                    \
         return Range<dimensions, T>::scaled(scaling);                       \
+    }                                                                       \
+    Type<T> scaledFromCenter(const VectorType<T>& scaling) const {          \
+        return Range<dimensions, T>::scaledFromCenter(scaling);             \
     }
 #endif
 
@@ -351,25 +362,41 @@ See @ref Range for more information.
 */
 template<class T> class Range2D: public Range<2, T> {
     public:
+        /**
+         * @brief Default constructor
+         *
+         * Equivalent to @ref Range2D(ZeroInitT).
+         */
+        constexpr /*implicit*/ Range2D() noexcept: Range<2, T>{ZeroInit} {}
+
         /** @copydoc Range(ZeroInitT) */
-        constexpr /*implicit*/ Range2D(ZeroInitT = ZeroInit) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<2, T>{ZeroInit}
-            #endif
-            {}
+        constexpr explicit Range2D(ZeroInitT) noexcept: Range<2, T>{ZeroInit} {}
 
         /** @copydoc Range(NoInitT) */
-        explicit Range2D(NoInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<2, T>{NoInit}
-            #endif
-            {}
+        explicit Range2D(Magnum::NoInitT) noexcept: Range<2, T>{Magnum::NoInit} {}
 
         /** @copydoc Range(const VectorType&, const VectorType&) */
         constexpr /*implicit*/ Range2D(const Vector2<T>& min, const Vector2<T>& max) noexcept: Range<2, T>(min, max) {}
 
+        /**
+         * @brief Construct a range from a pair of minimal and maximal coordinates
+         * @m_since{2020,06}
+         *
+         * Useful in combination with e.g. @ref minmax(), here for example to
+         * calculate texture bounds:
+         *
+         * @snippet MagnumMath.cpp Range-construct-minmax2D
+         *
+         * @todo std::pair constructors are not constexpr in C++11, make it so in C++14
+         */
+        /*implicit*/ Range2D(const std::pair<Vector2<T>, Vector2<T>>& minmax) noexcept: Range<2, T>{minmax.first, minmax.second} {}
+
+        /**
+         * @overload
+         * @m_since{2020,06}
+         */
+        /*implicit*/ Range2D(const std::pair<Vector<2, T>, Vector<2, T>>& minmax) noexcept: Range<2, T>{minmax.first, minmax.second} {}
+
         /** @copydoc Range(const Range<dimensions, U>&) */
         template<class U> constexpr explicit Range2D(const Range2D<U>& other) noexcept: Range<2, T>(other) {}
 
@@ -384,12 +411,7 @@ template<class T> class Range2D: public Range<2, T> {
             decltype(Implementation::RangeConverter<2, T, U>())
             #endif
             >
-        constexpr explicit Range2D(const U& other)
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<2, T>{Implementation::RangeConverter<2, T, U>::from(other)}
-            #endif
-            {}
+        constexpr explicit Range2D(const U& other): Range<2, T>{Implementation::RangeConverter<2, T, U>::from(other)} {}
 
         /** @copydoc Range(const Range&) */
         constexpr /*implicit*/ Range2D(const Range<2, T>& other) noexcept: Range<2, T>(other) {}
@@ -494,25 +516,41 @@ See @ref Range for more information.
 */
 template<class T> class Range3D: public Range<3, T> {
     public:
-        /** @copydoc Range(ZeroInitT) */
-        constexpr /*implicit*/ Range3D(ZeroInitT = ZeroInit) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<3, T>{ZeroInit}
-            #endif
-            {}
+        /**
+         * @brief Default constructor
+         *
+         * Equivalent to @ref Range3D(ZeroInitT).
+         */
+        constexpr /*implicit*/ Range3D() noexcept: Range<3, T>{ZeroInit} {}
 
-        /** @brief @copybrief Range(NoInitT) */
-        explicit Range3D(NoInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<3, T>{NoInit}
-            #endif
-            {}
+        /** @copydoc Range(ZeroInitT) */
+        constexpr explicit Range3D(ZeroInitT) noexcept: Range<3, T>{ZeroInit} {}
+
+        /** @brief @copybrief Range(Magnum::NoInitT) */
+        explicit Range3D(Magnum::NoInitT) noexcept: Range<3, T>{Magnum::NoInit} {}
 
         /** @copydoc Range(const VectorType&, const VectorType&) */
         constexpr /*implicit*/ Range3D(const Vector3<T>& min, const Vector3<T>& max) noexcept: Range<3, T>(min, max) {}
 
+        /**
+         * @brief Construct a range from a pair of minimal and maximal coordinates
+         * @m_since{2020,06}
+         *
+         * Useful in combination with e.g. @ref minmax(), here for example to
+         * calculate bounds of a triangle:
+         *
+         * @snippet MagnumMath.cpp Range-construct-minmax3D
+         *
+         * @todo std::pair constructors are not constexpr in C++11, make it so in C++14
+         */
+        /*implicit*/ Range3D(const std::pair<Vector3<T>, Vector3<T>>& minmax) noexcept: Range<3, T>{minmax.first, minmax.second} {}
+
+        /**
+         * @overload
+         * @m_since{2020,06}
+         */
+        /*implicit*/ Range3D(const std::pair<Vector<3, T>, Vector<3, T>>& minmax) noexcept: Range<3, T>{minmax.first, minmax.second} {}
+
         /** @copydoc Range(const Range<dimensions, U>&) */
         template<class U> constexpr explicit Range3D(const Range3D<U>& other) noexcept: Range<3, T>(other) {}
 
@@ -520,12 +558,7 @@ template<class T> class Range3D: public Range<3, T> {
          * @brief Construct range from external representation
          * @todoc Remove workaround when Doxygen no longer chokes on that line
          */
-        template<class U, class V = decltype(Implementation::RangeConverter<3, T, U>::from(std::declval<U>()))> constexpr explicit Range3D(const U& other) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Range<3, T>{Implementation::RangeConverter<3, T, U>::from(other)}
-            #endif
-            {}
+        template<class U, class V = decltype(Implementation::RangeConverter<3, T, U>::from(std::declval<U>()))> constexpr explicit Range3D(const U& other) noexcept: Range<3, T>{Implementation::RangeConverter<3, T, U>::from(other)} {}
 
         /** @copydoc Range(const Range&) */
         constexpr /*implicit*/ Range3D(const Range<3, T>& other) noexcept: Range<3, T>(other) {}
@@ -616,7 +649,7 @@ template<class T> class Range3D: public Range<3, T> {
         }
 
         /** @brief Range in the XY plane */
-        constexpr Range<2, T> xy() const {
+        constexpr Range2D<T> xy() const {
             return {Range<3, T>::min().xy(), Range<3, T>::max().xy()};
         }
 
@@ -724,6 +757,7 @@ template<UnsignedInt dimensions, class T> inline bool intersects(const Range<dim
            (Vector<dimensions, T>{a.min()} < b.max()).all();
 }
 
+#ifndef CORRADE_NO_DEBUG
 /** @debugoperator{Range} */
 template<UnsignedInt dimensions, class T> Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& debug, const Range<dimensions, T>& value) {
     debug << "Range({" << Corrade::Utility::Debug::nospace << Vector<dimensions, T>{value.min()}[0];
@@ -736,13 +770,6 @@ template<UnsignedInt dimensions, class T> Corrade::Utility::Debug& operator<<(Co
     return debug << Corrade::Utility::Debug::nospace << "})";
 }
 
-template<UnsignedInt dimensions, class T> inline bool Range<dimensions, T>::operator==(const Range<dimensions, T>& other) const {
-    /* For non-scalar types default implementation of TypeTraits would be used,
-       which is just operator== */
-    return TypeTraits<VectorType>::equals(_min, other._min) &&
-        TypeTraits<VectorType>::equals(_max, other._max);
-}
-
 /* Explicit instantiation for commonly used types */
 #ifndef DOXYGEN_GENERATING_OUTPUT
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const Range<1, Float>&);
@@ -755,6 +782,14 @@ extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utili
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const Range<2, Double>&);
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const Range<3, Double>&);
 #endif
+#endif
+
+template<UnsignedInt dimensions, class T> inline bool Range<dimensions, T>::operator==(const Range<dimensions, T>& other) const {
+    /* For non-scalar types default implementation of TypeTraits would be used,
+       which is just operator== */
+    return TypeTraits<VectorType>::equals(_min, other._min) &&
+        TypeTraits<VectorType>::equals(_max, other._max);
+}
 
 namespace Implementation {
 
@@ -773,41 +808,4 @@ template<UnsignedInt dimensions, class T> struct StrictWeakOrdering<Range<dimens
 
 }}
 
-namespace Corrade { namespace Utility {
-
-/** @configurationvalue{Magnum::Math::Range} */
-template<Magnum::UnsignedInt dimensions, class T> struct ConfigurationValue<Magnum::Math::Range<dimensions, T>> {
-    ConfigurationValue() = delete;
-
-    /** @brief Writes elements separated with spaces */
-    static std::string toString(const Magnum::Math::Range<dimensions, T>& value, const ConfigurationValueFlags flags) {
-        return ConfigurationValue<Magnum::Math::Vector<dimensions*2, T>>::toString(
-            reinterpret_cast<const Magnum::Math::Vector<dimensions*2, T>&>(value), flags);
-    }
-
-    /** @brief Reads elements separated with whitespace */
-    static Magnum::Math::Range<dimensions, T> fromString(const std::string& stringValue, const ConfigurationValueFlags flags) {
-        const auto vec = ConfigurationValue<Magnum::Math::Vector<dimensions*2, T>>::fromString(stringValue, flags);
-        return *reinterpret_cast<const Magnum::Math::Range<dimensions, T>*>(vec.data());
-    }
-};
-
-/** @configurationvalue{Magnum::Math::Range2D} */
-template<class T> struct ConfigurationValue<Magnum::Math::Range2D<T>>: public ConfigurationValue<Magnum::Math::Range<2, T>> {};
-
-/** @configurationvalue{Magnum::Math::Range3D} */
-template<class T> struct ConfigurationValue<Magnum::Math::Range3D<T>>: public ConfigurationValue<Magnum::Math::Range<3, T>> {};
-
-/* Explicit instantiation for commonly used types */
-#if !defined(DOXYGEN_GENERATING_OUTPUT) && !defined(__MINGW32__)
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<2, Magnum::Float>>;
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<2, Magnum::Int>>;
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<3, Magnum::Float>>;
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<3, Magnum::Int>>;
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<2, Magnum::Double>>;
-extern template struct MAGNUM_EXPORT ConfigurationValue<Magnum::Math::Range<3, Magnum::Double>>;
-#endif
-
-}}
-
 #endif
