@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -78,7 +78,10 @@ template<UnsignedInt dimensions, class T> class AbstractObject
     #endif
 {
     public:
-        /** @brief Transformation underlying type */
+        /**
+         * @brief Transformation underlying type
+         * @m_since{2019,10}
+         */
         typedef T Type;
 
         /** @brief Matrix type */
@@ -88,7 +91,11 @@ template<UnsignedInt dimensions, class T> class AbstractObject
         typedef AbstractFeature<dimensions, T> FeatureType;
 
         enum: UnsignedInt {
-            Dimensions = dimensions /**< Dimension count */
+            /**
+             * Dimension count
+             * @m_since{2019,10}
+             */
+            Dimensions = dimensions
         };
 
         /* This isn't protected because having virtual destructor here is
@@ -174,7 +181,11 @@ template<UnsignedInt dimensions, class T> class AbstractObject
             return doTransformationMatrices(objects, finalTransformationMatrix);
         }
 
-        /*@}*/
+        /* Since 1.8.17, the original short-hand group closing doesn't work
+           anymore. FFS. */
+        /**
+         * @}
+         */
 
         /**
          * @{ @name Transformation caching
@@ -233,7 +244,11 @@ template<UnsignedInt dimensions, class T> class AbstractObject
          */
         void setClean() { doSetClean(); }
 
-        /*@}*/
+        /* Since 1.8.17, the original short-hand group closing doesn't work
+           anymore. FFS. */
+        /**
+         * @}
+         */
 
     private:
         #ifndef DOXYGEN_GENERATING_OUTPUT /* https://bugzilla.gnome.org/show_bug.cgi?id=776986 */
@@ -298,7 +313,7 @@ typedef AbstractBasicObject3D<Float> AbstractObject3D;
 typedef AbstractObject<3, Float> AbstractObject3D;
 #endif
 
-#if defined(CORRADE_TARGET_WINDOWS) && !defined(__MINGW32__)
+#if defined(CORRADE_TARGET_WINDOWS) && !(defined(CORRADE_TARGET_MINGW) && !defined(CORRADE_TARGET_CLANG))
 extern template class MAGNUM_SCENEGRAPH_EXPORT AbstractObject<2, Float>;
 extern template class MAGNUM_SCENEGRAPH_EXPORT AbstractObject<3, Float>;
 #endif
