@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -72,7 +72,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Transform object
+         * @brief Transform the object
          * @return Reference to self (for method chaining)
          *
          * @see @ref transformLocal()
@@ -82,7 +82,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Transform object as a local transformation
+         * @brief Transform the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others.
@@ -92,7 +92,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Translate object
+         * @brief Translate the object
          * @return Reference to self (for method chaining)
          *
          * Same as calling @ref transform() with @ref Math::Matrix4::translation().
@@ -104,7 +104,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Translate object as a local transformation
+         * @brief Translate the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -115,22 +115,45 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object
+         * @brief Rotate the object using a quaternion
+         * @param quaternion    Normalized quaternion
+         * @return Reference to self (for method chaining)
+         * @m_since{2020,06}
+         *
+         * Expects that the quaternion is normalized.
+         * @see @ref rotate(Math::Rad<T>, const Math::Vector3<T>&),
+         *      @ref rotateLocal(const Math::Quaternion<T>&), @ref rotateX(),
+         *      @ref rotateY(), @ref rotateZ()
+         */
+        Object<BasicMatrixTransformation3D<T>>& rotate(const Math::Quaternion<T>& quaternion);
+
+        /**
+         * @brief Rotate the object using a quaternion as a local transformation
+         * @m_since{2020,06}
+         *
+         * Similar to the above, except that the transformation is applied
+         * before all others.
+         */
+        Object<BasicMatrixTransformation3D<T>>& rotateLocal(const Math::Quaternion<T>& quaternion);
+
+        /**
+         * @brief Rotate the object
          * @param angle             Angle (counterclockwise)
          * @param normalizedAxis    Normalized rotation axis
          * @return Reference to self (for method chaining)
          *
          * Same as calling @ref transform() with @ref Math::Matrix4::rotation().
-         * @see @ref rotateLocal(), @ref rotateX(), @ref rotateY(),
-         *      @ref rotateZ(), @ref Math::Vector3::xAxis(),
-         *      @ref Math::Vector3::yAxis(), @ref Math::Vector3::zAxis()
+         * @see @ref rotate(const Math::Quaternion<T>&), @ref rotateLocal(),
+         *      @ref rotateX(), @ref rotateY(), @ref rotateZ(),
+         *      @ref Math::Vector3::xAxis(), @ref Math::Vector3::yAxis(),
+         *      @ref Math::Vector3::zAxis()
          */
         Object<BasicMatrixTransformation3D<T>>& rotate(Math::Rad<T> angle, const Math::Vector3<T>& normalizedAxis) {
             return transform(Math::Matrix4<T>::rotation(angle, normalizedAxis));
         }
 
         /**
-         * @brief Rotate object as a local transformation
+         * @brief Rotate the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -141,7 +164,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around X axis
+         * @brief Rotate the object around X axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -153,7 +176,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around X axis as a local transformation
+         * @brief Rotate the object around X axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -164,7 +187,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around Y axis
+         * @brief Rotate the object around Y axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -176,7 +199,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around Y axis as a local transformation
+         * @brief Rotate the object around Y axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -187,7 +210,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around Z axis
+         * @brief Rotate the object around Z axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -199,7 +222,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Rotate object around Z axis as a local transformation
+         * @brief Rotate the object around Z axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -210,7 +233,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Scale object
+         * @brief Scale the object
          * @return Reference to self (for method chaining)
          *
          * Same as calling @ref transform() with @ref Math::Matrix4::scaling().
@@ -222,7 +245,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Scale object as a local transformation
+         * @brief Scale the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -233,7 +256,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Reflect object
+         * @brief Reflect the object
          * @param normal    Normal of the plane through which to reflect
          *      (normalized)
          * @return Reference to self (for method chaining)
@@ -246,7 +269,7 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         }
 
         /**
-         * @brief Reflect object as a local transformation
+         * @brief Reflect the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -266,6 +289,13 @@ template<class T> class BasicMatrixTransformation3D: public AbstractBasicTransla
         void doTranslate(const Math::Vector3<T>& vector) override final { translate(vector); }
         void doTranslateLocal(const Math::Vector3<T>& vector) override final { translateLocal(vector); }
 
+        void doRotate(const Math::Quaternion<T>& quaternion) override final {
+            rotate(quaternion);
+        }
+        void doRotateLocal(const Math::Quaternion<T>& quaternion) override final {
+            rotateLocal(quaternion);
+        }
+
         void doRotate(Math::Rad<T> angle, const Math::Vector3<T>& normalizedAxis) override final {
             rotate(angle, normalizedAxis);
         }
@@ -317,7 +347,8 @@ template<class T> struct Transformation<BasicMatrixTransformation3D<T>> {
 
 }
 
-#if defined(CORRADE_TARGET_WINDOWS) && !defined(__MINGW32__)
+#if defined(CORRADE_TARGET_WINDOWS) && !(defined(CORRADE_TARGET_MINGW) && !defined(CORRADE_TARGET_CLANG))
+extern template class MAGNUM_SCENEGRAPH_EXPORT BasicMatrixTransformation3D<Float>;
 extern template class MAGNUM_SCENEGRAPH_EXPORT Object<BasicMatrixTransformation3D<Float>>;
 #endif
 
